package com.sun.corba.se.impl.protocol;

import org.omg.CORBA.SystemException;
import com.sun.corba.se.spi.protocol.ForwardException;
import org.omg.CORBA.portable.ServantObject;
import org.omg.CORBA.Object;
import com.sun.corba.se.spi.oa.OADestroyed;
import com.sun.corba.se.spi.oa.OAInvocationInfo;
import com.sun.corba.se.spi.oa.ObjectAdapter;
import com.sun.corba.se.spi.ior.IOR;
import com.sun.corba.se.spi.orb.ORB;
import com.sun.corba.se.impl.logging.POASystemException;
import com.sun.corba.se.impl.logging.ORBUtilSystemException;

public class POALocalCRDImpl extends LocalClientRequestDispatcherBase
{
    private ORBUtilSystemException wrapper;
    private POASystemException poaWrapper;
    
    public POALocalCRDImpl(ORB orb, int n, IOR ior) {
        super(orb, n, ior);
        this.wrapper = ORBUtilSystemException.get(orb, "rpc.protocol");
        this.poaWrapper = POASystemException.get(orb, "rpc.protocol");
    }
    
    private OAInvocationInfo servantEnter(ObjectAdapter objectAdapter) throws OADestroyed {
        objectAdapter.enter();
        OAInvocationInfo invocationInfo = objectAdapter.makeInvocationInfo(this.objectId);
        this.orb.pushInvocationInfo(invocationInfo);
        return invocationInfo;
    }
    
    private void servantExit(ObjectAdapter objectAdapter) {
        try {
            objectAdapter.returnServant();
        }
        finally {
            objectAdapter.exit();
            this.orb.popInvocationInfo();
        }
    }
    
    public ServantObject servant_preinvoke(org.omg.CORBA.Object object, String operation, Class clazz) {
        ObjectAdapter find = this.oaf.find(this.oaid);
        OAInvocationInfo servantEnter;
        try {
            servantEnter = this.servantEnter(find);
            servantEnter.setOperation(operation);
        }
        catch (OADestroyed oaDestroyed) {
            return this.servant_preinvoke(object, operation, clazz);
        }
        try {
            try {
                find.getInvocationServant(servantEnter);
                if (!this.checkForCompatibleServant(servantEnter, clazz)) {
                    return null;
                }
            }
            catch (Throwable t) {
                this.servantExit(find);
                throw t;
            }
        }
        catch (ForwardException cause) {
            RuntimeException ex = new RuntimeException("deal with this.");
            ex.initCause(cause);
            throw ex;
        }
        catch (ThreadDeath threadDeath) {
            throw this.wrapper.runtimeexception(threadDeath);
        }
        catch (Throwable t2) {
            if ((t2 instanceof SystemException)) {
                throw (SystemException)t2;
            }
            throw this.poaWrapper.localServantLookup(t2);
        }
        if (!this.checkForCompatibleServant(servantEnter, clazz)) {
            this.servantExit(find);
            return null;
        }
        return servantEnter;
    }
    
    public void servant_postinvoke(org.omg.CORBA.Object object, ServantObject servantObject) {
        this.servantExit(this.orb.peekInvocationInfo().oa());
    }
}
